@@ -1,20 +1,23 @@
 from uuid import uuid4
+import logging
 from django.conf import settings
 from django.db import connection
+import ddtrace
 from .utils import *
 
 
 __all__ = [
-    "ErrorReportingMiddleware"
+    "ErrorReportingMiddleware", "DataDogExceptionMiddleware",
 ]
 
+logger = logging.getLogger(__name__)
+
 
 class ErrorReportingMiddleware(object):
     def __init__(self, get_response):
         self.get_response = get_response
         try:
-            from ddtrace import tracer
-            self.dd_scope = tracer.current_span()
+            self.dd_scope = ddtrace.tracer.current_root_span()
         except:
             self.dd_scope = None
 
@@ -83,11 +86,53 @@ class ErrorReportingMiddleware(object):
                 self.dd_scope
             )
 
-        if hasattr(settings, "ERROR_REPORTING_TAGGING_CALLBACK"):
-            settings.ERROR_REPORTING_TAGGING_CALLBACK(
+        add_event_tag(
+            "http.client.ip_address",
+            request.META.get("REMOTE_ADDR"),
+            self.dd_scope
+        )
+
+        add_event_tag(
+            "http.client.user_agent",
+            request.META.get("HTTP_USER_AGENT"),
+            self.dd_scope
+        )
+
+        if hasattr(settings, "DER_REQUEST_TAGGING_CB") and settings.DER_REQUEST_TAGGING_CB:
+            settings.DER_REQUEST_TAGGING_CB(
                 request,
                 add_event_tag,
                 self.dd_scope
             )
 
         return self.get_response(request)
+
+
+class DataDogExceptionMiddleware(object):
+    """
+    When an exception is captured, this middleware will set the appropriate span tags on the root span of the trace.
+
+    This middleware should be added as late as possible.
+    """
+    def __init__(self, get_response):
+        self.get_response = get_response
+
+    def __call__(self, request):
+        response = self.get_response(request)
+        return response
+
+    def process_exception(self, request, exception):
+        if exception:
+            root_span = ddtrace.tracer.current_root_span()
+
+            if root_span:
+                root_span.set_exc_info(
+                    type(exception),
+                    exception,
+                    exception.__traceback__
+                )
+            else:
+                logger.warning("Could not get DataDog root span")
+
+        # [!!] Return nothing so other middleware will process
+
